@@ -5,13 +5,13 @@ produce an address space where that program is loaded and ready to run.
 The primary interface to CLE is the Loader class.
 """
 
-__version__ = (8, 19, 7, 25)
+__version__ = (8, 20, 1, 7)
 
 if bytes is str:
     raise Exception("This module is designed for python 3 only. Please install an older version to use python 2.")
 
 import logging
-logging.getLogger("cle").addHandler(logging.NullHandler())
+logging.getLogger(name=__name__).addHandler(logging.NullHandler())
 
 # pylint: disable=wildcard-import
 from . import utils
