@@ -1,6 +1,5 @@
 import os
 import struct
-import subprocess
 import logging
 import archinfo
 import elftools
@@ -19,7 +18,7 @@ from ...errors import CLEError, CLEInvalidBinaryError, CLECompatibilityError
 from ...utils import ALIGN_DOWN, ALIGN_UP, get_mmaped_data, stream_or_path
 from ...address_translator import AT
 
-l = logging.getLogger('cle.elf')
+l = logging.getLogger(name=__name__)
 
 __all__ = ('ELFSymbol', 'ELF')
 
@@ -62,6 +61,10 @@ class ELF(MetaELF):
         # Get an appropriate archinfo.Arch for this binary, unless the user specified one
         if self.arch is None:
             self.set_arch(self.extract_arch(self.reader))
+        else:
+            other_arch = self.extract_arch(self.reader)
+            if other_arch != self.arch:
+                l.warning("User specified %s but autodetected %s. Proceed with caution.", self.arch, other_arch)
 
         self._addend = addend
 
@@ -85,7 +88,6 @@ class ELF(MetaELF):
         self._symbol_cache = {}
         self._symbols_by_name = {}
         self._desperate_for_symbols = False
-        self.demangled_names = {}
         self.imports = {}
         self.resolved_imports = []
 
@@ -120,8 +122,6 @@ class ELF(MetaELF):
         self._ppc64_abiv1_entry_fix()
         self._load_plt()
 
-        self._populate_demangled_names()
-
         for offset, patch in patch_undo:
             self.memory.store(AT.from_lva(self.min_addr + offset, self).to_rva(), patch)
 
@@ -248,7 +248,7 @@ class ELF(MetaELF):
                 return self._nullsymbol
             try:
                 re_sym = symbol_table.get_symbol(symid)
-            except Exception: # pylint: disable=bare-except
+            except Exception:  # pylint: disable=broad-except
                 l.exception("Error parsing symbol at %#08x", symid)
                 return None
             cache_key = self._symbol_to_tuple(re_sym)
@@ -400,29 +400,6 @@ class ELF(MetaELF):
 
         return RelocClass(self, symbol, address, addend)
 
-    def _populate_demangled_names(self):
-        """
-        TODO: remove this once a python implementation of a name demangler has
-        been implemented, then update self.demangled_names in Symbol
-        """
-
-        if not self.symbols:
-            return
-
-        names = [s.name for s in self.symbols if s.name.startswith("_Z")]
-        # this monstrosity taken from stackoverflow
-        # http://stackoverflow.com/questions/6526500/c-name-mangling-library-for-python
-        args = ['c++filt']
-        args.extend(n.split('@@')[0] for n in names)
-        try:
-            pipe = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
-            stdout, _ = pipe.communicate()
-            demangled = stdout.decode().split("\n")[:-1]
-
-            self.demangled_names = dict(zip(names, demangled))
-        except OSError:
-            pass
-
     #
     # Private Methods... really. Calling these out of context
     # will probably break things. Caveat emptor.
