@@ -17,7 +17,7 @@ except ImportError:
 
 __all__ = ('Loader',)
 
-l = logging.getLogger("cle.loader")
+l = logging.getLogger(name=__name__)
 
 
 class Loader:
@@ -52,6 +52,10 @@ class Loader:
     :param page_size:           The granularity with which data is mapped into memory. Set to 1 if you are working
                                 in a non-paged environment.
     :param preload_libs:        Similar to `force_load_libs` but will provide for symbol resolution, with precedence
+    :param extern_size:         How much space to allocate for externs. Default 0x8000, or 0x80000 for 64-bit
+                                architectures.
+    :param tls_size:            How much space to allocate for tls. Default 0x8000, or 0x80000 for 64-bit
+                                architectures.
                                 over any dependencies.
     :ivar memory:               The loaded, rebased, and relocated memory of the program.
     :vartype memory:            cle.memory.Clemory
@@ -77,7 +81,7 @@ class Loader:
                  main_opts=None, lib_opts=None, ld_path=(), use_system_libs=True,
                  ignore_import_version_numbers=True, case_insensitive=False, rebase_granularity=0x1000000,
                  except_missing_libs=False, aslr=False, perform_relocations=True,
-                 page_size=0x1, extern_size=0x8000, preload_libs=(), arch=None):
+                 page_size=0x1, extern_size=None, tls_size=None, preload_libs=(), arch=None):
         if hasattr(main_binary, 'seek') and hasattr(main_binary, 'read'):
             self._main_binary_path = None
             self._main_binary_stream = main_binary
@@ -94,12 +98,15 @@ class Loader:
         self._main_opts = {} if main_opts is None else main_opts
         self._lib_opts = {} if lib_opts is None else lib_opts
         self._custom_ld_path = [ld_path] if type(ld_path) is str else ld_path
+        force_load_libs = [force_load_libs] if type(force_load_libs) is str else force_load_libs
+        preload_libs = [preload_libs] if type(preload_libs) is str else preload_libs
         self._use_system_libs = use_system_libs
         self._ignore_import_version_numbers = ignore_import_version_numbers
         self._case_insensitive = case_insensitive
         self._rebase_granularity = rebase_granularity
         self._except_missing_libs = except_missing_libs
         self._extern_size = extern_size
+        self._tls_size = tls_size
         self._relocated_objects = set()
         self._perform_relocations = perform_relocations
 
@@ -198,9 +205,13 @@ class Loader:
         Accessing this property will load this object into memory if it was not previously present.
         """
         if self._extern_object is None:
-            if self.main_object.arch.bits < 32 and self._extern_size == 0x8000:
-                l.warning("Your extern object is probably too big for your memory space.  Making it 0x200")
-                self._extern_size = 0x200
+            if self._extern_size is None:
+                if self.main_object.arch.bits < 32:
+                    self._extern_size = 0x200
+                elif self.main_object.arch.bits == 32:
+                    self._extern_size = 0x8000
+                else:
+                    self._extern_size = 0x80000
             self._extern_object = ExternObject(self, map_size=self._extern_size)
             self._map_object(self._extern_object)
         return self._extern_object
@@ -225,11 +236,18 @@ class Loader:
         Accessing this property will load this object into memory if it was not previously present.
         """
         if self._tls_object is None:
+            if self._tls_size is None:
+                if self.main_object.arch.bits < 32:
+                    self._tls_size = 0x200
+                elif self.main_object.arch.bits == 32:
+                    self._tls_size = 0x8000
+                else:
+                    self._tls_size = 0x80000
             if isinstance(self.main_object, MetaELF):
-                self._tls_object = ELFTLSObject(self)
+                self._tls_object = ELFTLSObject(self, max_data=self._tls_size)
                 self._map_object(self._tls_object)
             elif isinstance(self.main_object, PE):
-                self._tls_object = PETLSObject(self)
+                self._tls_object = PETLSObject(self, max_data=self._tls_size)
                 self._map_object(self._tls_object)
         return self._tls_object
 
@@ -623,7 +641,7 @@ class Loader:
     def _internal_load(self, *args, preloading=False):
         """
         Pass this any number of files or libraries to load. If it can't load any of them for any reason, it will
-        except out. Note that the sematics of ``auto_load_libs`` and ``except_missing_libs`` apply at all times.
+        except out. Note that the semantics of ``auto_load_libs`` and ``except_missing_libs`` apply at all times.
 
         It will return a list of all the objects successfully loaded, which may be smaller than the list you provided
         if any of them were previously loaded.
@@ -814,7 +832,7 @@ class Loader:
         overlap with anything already loaded.
         """
         # this assumes that self.main_object exists, which should... definitely be safe
-        if self.main_object.arch.bits < 32:
+        if self.main_object.arch.bits < 32 or self.main_object.max_addr >= 2**(self.main_object.arch.bits-1):
             # HACK: On small arches, we should be more aggressive in packing stuff in.
             gap_start = 0
         else:
@@ -994,7 +1012,7 @@ class Loader:
                 if self._ignore_import_version_numbers:
                     yield os.path.basename(spec.binary).rstrip('.0123456789')
         elif hasattr(spec, 'read') and hasattr(spec, 'seek'):
-            backend_cls = self._static_backend(spec)
+            backend_cls = self._static_backend(spec, ignore_hints=True)
             if backend_cls is not None:
                 soname = backend_cls.extract_soname(spec)
                 if soname is not None:
@@ -1009,7 +1027,7 @@ class Loader:
                 yield os.path.basename(spec).rstrip('.0123456789')
 
             if os.path.exists(spec):
-                backend_cls = self._static_backend(spec)
+                backend_cls = self._static_backend(spec, ignore_hints=True)
                 if backend_cls is not None:
                     soname = backend_cls.extract_soname(spec)
                     if soname is not None:
@@ -1021,17 +1039,19 @@ class Loader:
             for name in self._possible_idents(spec, lowercase=True):
                 yield name.lower()
 
-    def _static_backend(self, spec):
+    def _static_backend(self, spec, ignore_hints=False):
         """
         Returns the correct loader for the file at `spec`.
         Returns None if it's a blob or some unknown type.
         TODO: Implement some binwalk-like thing to carve up blobs automatically
         """
 
-        try:
-            return self._backend_resolver(self._lib_opts[spec]['backend'])
-        except KeyError:
-            pass
+        if not ignore_hints:
+            for ident in self._possible_idents(spec):
+                try:
+                    return self._backend_resolver(self._lib_opts[ident]['backend'])
+                except KeyError:
+                    pass
 
         with stream_or_path(spec) as stream:
             for rear in ALL_BACKENDS.values():
